@@ -1,22 +1,51 @@
-from .structures import CmdException, CmdResult
-from .utils import cmd2args, indent
+import os
 import subprocess
 import sys
 import tempfile
-import os
-import warnings
+
+import six
+from contracts import contract
+
+from . import logger
+from .structures import CmdException, CmdResult
+from .utils import cmd2args, copyable_cmd, indent
+
+# import signal
 
 
-__all__ = ['system_cmd_result']
+__all__ = [
+    'system_cmd_result',
+]
 
+
+class Shared(object):
+    p = None
+
+
+#
+# def on_sigterm(a,b):
+#     #print('SIGTERM caught --- terminating child process')
+#     try:
+#         Shared.p.terminate()
+#     except OSError: # e.g. OSError: [Errno 3] No such process
+#         pass   
+#     #os.kill(Shared.p.pid, signal.SIGKILL)
+# 
+# def set_term_function(process):
+#     Shared.p = process
+#     signal.signal(signal.SIGTERM, on_sigterm)
+
+
+@contract(cwd='None|str', cmd='str|list(str)', env='dict|None')
 def system_cmd_result(cwd, cmd,
                       display_stdout=False,
                       display_stderr=False,
                       raise_on_error=False,
-                      display_prefix=None,
+                      display_prefix=None,  # leave it there
                       write_stdin='',
                       capture_keyboard_interrupt=False,
-                      display_stream=sys.stderr):  # @UnusedVariable
+                      display_stream=sys.stdout,  # @UnusedVariable
+                      env=None):
     ''' 
         Returns the structure CmdResult; raises CmdException.
         Also OSError are captured.
@@ -24,9 +53,10 @@ def system_cmd_result(cwd, cmd,
         
         :param write_stdin: A string to write to the process.
     '''
-    # if display_prefix is not None:
-        # warnings.warn('call with display_prefix')
-        
+
+    if env is None:
+        env = os.environ.copy()
+
     tmp_stdout = tempfile.TemporaryFile()
     tmp_stderr = tempfile.TemporaryFile()
 
@@ -34,18 +64,28 @@ def system_cmd_result(cwd, cmd,
     rets = None
     interrupted = False
 
+    #     if (display_stdout and captured_stdout) or (display_stderr and captured_stderr):
+
     try:
-        #stdout = None if display_stdout else 
+        # stdout = None if display_stdout else 
         stdout = tmp_stdout.fileno()
-#        stderr = None if display_stderr else 
+        # stderr = None if display_stderr else 
         stderr = tmp_stderr.fileno()
+        if isinstance(cmd, str):
+            cmd = cmd2args(cmd)
+
+        assert isinstance(cmd, list)
+        if display_stdout or display_stderr:
+            logger.info('$ %s' % copyable_cmd(cmd))
         p = subprocess.Popen(
-                cmd2args(cmd),
+                cmd,
                 stdin=subprocess.PIPE,
                 stdout=stdout,
                 stderr=stderr,
                 bufsize=0,
-                cwd=cwd)
+                cwd=cwd,
+                env=env)
+        #         set_term_function(p)
 
         if write_stdin != '':
             p.stdin.write(write_stdin)
@@ -58,11 +98,12 @@ def system_cmd_result(cwd, cmd,
         interrupted = False
 
     except KeyboardInterrupt:
+        logger.debug('Keyboard interrupt for:\n %s' % " ".join(cmd))
         if capture_keyboard_interrupt:
             ret = 100
             interrupted = True
         else:
-            raise 
+            raise
     except OSError as e:
         interrupted = False
         ret = 200
@@ -73,15 +114,26 @@ def system_cmd_result(cwd, cmd,
         os.lseek(f.fileno(), 0, 0)
         return f.read().strip()
 
-    captured_stdout = read_all(tmp_stdout)
-    captured_stderr = read_all(tmp_stderr)
+    captured_stdout = read_all(tmp_stdout).strip()
+    captured_stderr = read_all(tmp_stderr).strip()
+
+    s = ""
+
+    captured_stdout = remove_empty_lines(captured_stdout)
+    captured_stderr = remove_empty_lines(captured_stderr)
+
+    if six.PY3:
+        captured_stdout = captured_stdout.decode()
+        captured_stderr = captured_stderr.decode()
 
     if display_stdout and captured_stdout:
-        display_stream.write(indent(captured_stdout,'stderr>'))
+        s += indent(captured_stdout, 'stdout>') + '\n'
 
     if display_stderr and captured_stderr:
-        display_stream.write(indent(captured_stderr,'stdout>'))
+        s += indent(captured_stderr, 'stderr>') + '\n'
 
+    if s:
+        logger.debug(s)
 
     res = CmdResult(cwd, cmd, ret, rets, interrupted,
                     stdout=captured_stdout,
@@ -93,7 +145,15 @@ def system_cmd_result(cwd, cmd,
 
     return res
 
-# 
+
+def remove_empty_lines(s):
+    lines = s.split(b"\n")
+    empty = lambda line: len(line.strip()) == 0
+    lines = [l for l in lines if not empty(l)]
+    return b"\n".join(lines)
+
+
+#
 # def system_cmd_result(
 #     cwd, cmd,
 #     display_stdout=False,
@@ -158,11 +218,11 @@ def system_cmd_result(cwd, cmd,
 #             raise CmdException(res)
 #     
 #     return res
-        
-        
+
+
 def alternative_nonworking(p, display_stderr, display_stdout, display_prefix):
     """ Returns stdout, stderr """
-    
+
     # p.stdin.close()
     stderr = ''
     stdout = ''
@@ -170,7 +230,7 @@ def alternative_nonworking(p, display_stderr, display_stdout, display_prefix):
     stdout_lines = []
     stderr_to_read = True
     stdout_to_read = True
-    
+
     def read_stream(stream, lines):
         if stream:
             nexti = stream.readline()
@@ -182,28 +242,28 @@ def alternative_nonworking(p, display_stderr, display_stdout, display_prefix):
         else:
             stream.close()
             return False
-            
+
     # XXX: read all the lines
     while stderr_to_read or stdout_to_read:
-        
+
         if stderr_to_read:
             stderr_to_read = read_stream(p.stderr, stderr_lines)
-#             stdout_to_read = False
-    
+        #             stdout_to_read = False
+
         if stdout_to_read:
             stdout_to_read = read_stream(p.stdout, stdout_lines)
-        
+
         while stderr_lines:
             l = stderr_lines.pop(0)
             stderr += l
             if display_stderr:
                 sys.stderr.write('%s ! %s' % (display_prefix, l))
-                
+
         while stdout_lines:
             l = stdout_lines.pop(0)
             stdout += l
             if display_stdout:
                 sys.stderr.write('%s   %s' % (display_prefix, l))
-            
+
     stdout = p.stdout.read()
     return stdout, stderr
