@@ -1,19 +1,9 @@
 from .utils import indent
 
-__all__ = ['CmdResult', 'CmdException']
-
-# class CmdResult(object):
-#     def __init__(self, cwd, cmd, ret, stdout, stderr):
-#         self.cwd = cwd
-#         self.cmd = cmd
-#         self.ret = ret
-#         self.stdout = stdout
-#         self.stderr = stderr
-#         
-#     def format(self):
-#         return result_format(self.cwd, self.cmd, self.ret, self.stdout, self.stderr)
-
-
+__all__ = [
+    'CmdResult', 
+    'CmdException',
+] 
 
 class CmdResult(object):
     def __init__(self, cwd, cmd, ret, rets, interrupted, stdout, stderr):
@@ -26,8 +16,9 @@ class CmdResult(object):
         self.interrupted = interrupted
 
     def __str__(self):
+        from system_cmd.utils import copyable_cmd
         msg = ('The command: %s\n'
-               '     in dir: %s\n' % (self.cmd, self.cwd))
+               '     in dir: %s\n' % (copyable_cmd(self.cmd), self.cwd))
 
         if self.interrupted:
             msg += 'Was interrupted by the user\n'
